@@ -67,6 +67,7 @@ struct a2dp_data {
 
 struct headset_data {
 	headset_lock_t lock;
+	gboolean locked;
 };
 
 struct unix_client {
@@ -94,6 +95,9 @@ static void client_free(struct unix_client *client)
 {
 	struct a2dp_data *a2dp;
 
+	if (client->req_id)
+		client->cancel(client->dev, client->req_id);
+
 	switch (client->type) {
 	case TYPE_SINK:
 	case TYPE_SOURCE:
@@ -250,6 +254,8 @@ static void headset_discovery_complete(struct audio_device *dev, void *user_data
 	pcm->sampling_rate = 8000;
 	if (headset_get_nrec(dev))
 		pcm->flags |= BT_PCM_FLAG_NREC;
+	if (!headset_get_sco_hci(dev))
+		pcm->flags |= BT_PCM_FLAG_PCM_ROUTING;
 
 	rsp->h.type = BT_RESPONSE;
 	rsp->h.name = BT_GET_CAPABILITIES;
@@ -309,7 +315,9 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	if (!dev)
 		goto failed;
 
-	if (!headset_lock(dev, hs->lock)) {
+	if (!hs->locked)
+		hs->locked = headset_lock(dev, hs->lock);
+	if (!hs->locked) {
 		error("Unable to lock headset");
 		goto failed;
 	}
@@ -317,7 +325,8 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	client->data_fd = headset_get_sco_fd(dev);
 	if (client->data_fd < 0) {
 		error("Unable to get a SCO fd");
-		headset_unlock(client->dev, hs->lock);
+		headset_unlock(dev, hs->lock);
+		hs->locked = FALSE;
 		goto failed;
 	}
 
@@ -338,6 +347,7 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	if (unix_sendmsg_fd(client->sock, client->data_fd) < 0) {
 		error("unix_sendmsg_fd: %s(%d)", strerror(errno), errno);
 		headset_unlock(client->dev, hs->lock);
+		hs->locked = FALSE;
 		goto failed;
 	}
 
@@ -352,7 +362,7 @@ static void headset_suspend_complete(struct audio_device *dev, void *user_data)
 {
 	struct unix_client *client = user_data;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
-	struct bt_start_stream_rsp *rsp = (void *) buf;
+	struct bt_stop_stream_rsp *rsp = (void *) buf;
 
 	if (!dev)
 		goto failed;
@@ -613,7 +623,7 @@ static void a2dp_suspend_complete(struct avdtp *session,
 {
 	struct unix_client *client = user_data;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
-	struct bt_start_stream_rsp *rsp = (void *) buf;
+	struct bt_stop_stream_rsp *rsp = (void *) buf;
 	struct a2dp_data *a2dp = &client->d.a2dp;
 
 	if (err)
@@ -648,8 +658,6 @@ static void start_discovery(struct audio_device *dev, struct unix_client *client
 	struct a2dp_data *a2dp;
 	int err = 0;
 
-	client->type = select_service(dev, client->interface);
-
 	switch (client->type) {
 	case TYPE_SINK:
 		a2dp = &client->d.a2dp;
@@ -691,8 +699,6 @@ static void start_config(struct audio_device *dev, struct unix_client *client)
 	struct headset_data *hs;
 	unsigned int id;
 
-	client->type = select_service(dev, client->interface);
-
 	switch (client->type) {
 	case TYPE_SINK:
 		a2dp = &client->d.a2dp;
@@ -758,8 +764,6 @@ static void start_resume(struct audio_device *dev, struct unix_client *client)
 	struct headset_data *hs;
 	unsigned int id;
 
-	client->type = select_service(dev, client->interface);
-
 	switch (client->type) {
 	case TYPE_SINK:
 		a2dp = &client->d.a2dp;
@@ -801,6 +805,9 @@ static void start_resume(struct audio_device *dev, struct unix_client *client)
 		goto failed;
 	}
 
+	client->req_id = id;
+	client->dev = dev;
+
 	return;
 
 failed:
@@ -813,8 +820,6 @@ static void start_suspend(struct audio_device *dev, struct unix_client *client)
 	struct headset_data *hs;
 	unsigned int id;
 
-	client->type = select_service(dev, client->interface);
-
 	switch (client->type) {
 	case TYPE_SINK:
 		a2dp = &client->d.a2dp;
@@ -870,8 +875,8 @@ static void handle_getcapabilities_req(struct unix_client *client,
 	str2ba(req->device, &bdaddr);
 
 	if (client->interface) {
-		g_free(client->interface);
-		client->interface = NULL;
+		error("Got GET_CAPABILITIES for an initialized client");
+		goto failed;
 	}
 
 	if (req->transport == BT_CAPABILITIES_TRANSPORT_SCO)
@@ -883,15 +888,19 @@ static void handle_getcapabilities_req(struct unix_client *client,
 		goto failed;
 
 	dev = manager_find_device(&bdaddr, client->interface, TRUE);
+	if (!dev && (req->flags & BT_FLAG_AUTOCONNECT))
+		dev = manager_find_device(&bdaddr, client->interface, FALSE);
+
 	if (!dev) {
-		if (req->flags & BT_FLAG_AUTOCONNECT)
-			dev = manager_find_device(&bdaddr, client->interface, FALSE);
-		else
-			goto failed;
+		error("Unable to find a matching device");
+		goto failed;
 	}
 
-	if (!dev)
+	client->type = select_service(dev, client->interface);
+	if (client->type == TYPE_NONE) {
+		error("No matching service found");
 		goto failed;
+	}
 
 	start_discovery(dev, client);
 
@@ -904,7 +913,10 @@ failed:
 static int handle_sco_transport(struct unix_client *client,
 				struct bt_set_configuration_req *req)
 {
-	client->interface = g_strdup(AUDIO_HEADSET_INTERFACE);
+	if (!client->interface)
+		client->interface = g_strdup(AUDIO_HEADSET_INTERFACE);
+	else if (!g_str_equal(client->interface, AUDIO_HEADSET_INTERFACE))
+		return -EIO;
 
 	debug("config sco - device = %s access_mode = %u", req->device,
 			req->access_mode);
@@ -919,7 +931,10 @@ static int handle_a2dp_transport(struct unix_client *client,
 	struct sbc_codec_cap sbc_cap;
 	struct mpeg_codec_cap mpeg_cap;
 
-	client->interface = g_strdup(AUDIO_SINK_INTERFACE);
+	if (!client->interface)
+		client->interface = g_strdup(AUDIO_SINK_INTERFACE);
+	else if (!g_str_equal(client->interface, AUDIO_SINK_INTERFACE))
+		return -EIO;
 
 	if (client->caps) {
 		g_slist_foreach(client->caps, (GFunc) g_free, NULL);
@@ -1003,11 +1018,6 @@ static void handle_setconfiguration_req(struct unix_client *client,
 
 	str2ba(req->device, &bdaddr);
 
-	if (client->interface) {
-		g_free(client->interface);
-		client->interface = NULL;
-	}
-
 	if (req->codec.transport == BT_CAPABILITIES_TRANSPORT_SCO) {
 		err = handle_sco_transport(client, req);
 		if (err < 0) {
@@ -1102,8 +1112,10 @@ static gboolean client_cb(GIOChannel *chan, GIOCondition cond, gpointer data)
 		debug("Unix client disconnected (fd=%d)", client->sock);
 		switch (client->type) {
 		case TYPE_HEADSET:
-			if (client->dev)
+			if (client->dev && hs->locked) {
 				headset_unlock(client->dev, hs->lock);
+				hs->locked = FALSE;
+			}
 			break;
 		case TYPE_SOURCE:
 		case TYPE_SINK:
