@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -189,10 +189,19 @@ static void parse_config(GKeyFile *config)
 		g_clear_error(&err);
 	} else {
 		debug("deviceid=%s", str);
-		strncpy(main_opts.deviceid, str, sizeof(main_opts.deviceid));
+		strncpy(main_opts.deviceid, str,
+					sizeof(main_opts.deviceid) - 1);
 		g_free(str);
 	}
 
+	boolean = g_key_file_get_boolean(config, "General",
+						"ReverseServiceDiscovery", &err);
+	if (err) {
+		debug("%s", err->message);
+		g_clear_error(&err);
+	} else
+		main_opts.reverse_sdp = boolean;
+
 	main_opts.link_mode = HCI_LM_ACCEPT;
 
 	main_opts.link_policy = HCI_LP_RSWITCH | HCI_LP_SNIFF |
@@ -404,6 +413,8 @@ static void configure_device(int dev_id)
 	policy = htobs(main_opts.link_policy);
 	hci_send_cmd(dd, OGF_LINK_POLICY,
 				OCF_WRITE_DEFAULT_LINK_POLICY, 2, &policy);
+
+	hci_close_dev(dd);
 }
 
 static void init_device(int dev_id)
@@ -480,6 +491,8 @@ static void device_devreg_setup(int dev_id, gboolean devup)
 
 	init_device(dev_id);
 
+	memset(&di, 0, sizeof(di));
+
 	if (hci_devinfo(dev_id, &di) < 0)
 		return;
 
@@ -545,6 +558,7 @@ static void init_defaults(void)
 	main_opts.name	= g_strdup("BlueZ");
 	main_opts.discovto	= HCID_DEFAULT_DISCOVERABLE_TIMEOUT;
 	main_opts.remember_powered = TRUE;
+	main_opts.reverse_sdp = TRUE;
 
 	if (gethostname(main_opts.host_name, sizeof(main_opts.host_name) - 1) < 0)
 		strcpy(main_opts.host_name, "noname");
@@ -578,7 +592,8 @@ static inline void device_event(GIOChannel *chan, evt_stack_internal *si)
 	}
 }
 
-static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data)
+static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond,
+								gpointer data)
 {
 	unsigned char buf[HCI_MAX_FRAME_SIZE], *ptr;
 	evt_stack_internal *si;
@@ -589,7 +604,8 @@ static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer dat
 
 	ptr = buf;
 
-	if ((err = g_io_channel_read(chan, (gchar *) buf, sizeof(buf), &len))) {
+	err = g_io_channel_read(chan, (gchar *) buf, sizeof(buf), &len);
+	if (err) {
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
 
@@ -699,9 +715,10 @@ int main(int argc, char *argv[])
 	}
 
 	/* Create and bind HCI socket */
-	if ((main_opts.sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
-		error("Can't open HCI socket: %s (%d)",
-							strerror(errno), errno);
+	main_opts.sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
+	if (main_opts.sock < 0) {
+		error("Can't open HCI socket: %s (%d)", strerror(errno),
+								errno);
 		exit(1);
 	}
 
@@ -709,15 +726,17 @@ int main(int argc, char *argv[])
 	hci_filter_clear(&flt);
 	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
 	hci_filter_set_event(EVT_STACK_INTERNAL, &flt);
-	if (setsockopt(main_opts.sock, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		error("Can't set filter: %s (%d)",
-							strerror(errno), errno);
+	if (setsockopt(main_opts.sock, SOL_HCI, HCI_FILTER, &flt,
+							sizeof(flt)) < 0) {
+		error("Can't set filter: %s (%d)", strerror(errno), errno);
 		exit(1);
 	}
 
+	memset(&addr, 0, sizeof(addr));
 	addr.hci_family = AF_BLUETOOTH;
 	addr.hci_dev = HCI_DEV_NONE;
-	if (bind(main_opts.sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
+	if (bind(main_opts.sock, (struct sockaddr *) &addr,
+							sizeof(addr)) < 0) {
 		error("Can't bind HCI socket: %s (%d)",
 							strerror(errno), errno);
 		exit(1);
